﻿using CallCenter.Calls;
using CallCenter.Share.Dtos;
using CallCenter.Share.Enums;
using CallCenter.Share.Requests;
using MapsterMapper;
using MessagePack.Formatters;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Options;

namespace CallCenter.Api.Controllers
{
    /// <summary>
    /// 
    /// </summary>
    public class CallController : BaseController
    {
        private readonly ICallDomainService _callDomainService;
        private readonly ICallRepository _callRepository;
        private readonly IMapper _mapper;
        private readonly ICallDetailRepository _callDetailRepository;

        public CallController(ICallDomainService callDomainService, ICallRepository callRepository, IMapper mapper, ICallDetailRepository callDetailRepository)
        {
            _callDomainService = callDomainService;
            _callRepository = callRepository;
            _mapper = mapper;
            _callDetailRepository = callDetailRepository;
        }


        #region 强拆

        /// <summary>
        /// 强拆分机
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("ClearExt")]
        public async Task ClearExt([FromBody] ClearExtRequest request)
        {
            await _callDomainService.ClearExtAsync(request, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 强拆来电
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("ClearVisitor")]
        public async Task ClearVisitor([FromBody] ClearVisitorRequest request)
        {
            await _callDomainService.ClearVisitorAsync(request, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 强拆去电
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpPost("ClearOuter")]
        public async Task ClearOuter([FromBody] ClearOuterRequest request)
        {
            await _callDomainService.ClearOuterAsync(request, HttpContext.RequestAborted);
        }

        #endregion

        #region 通话记录

        /// <summary>
        /// 分页通话记录列表
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpGet("paged")]
        public async Task<PagedDto<CallDto>> GetCallList([FromQuery]GetCallListRequest request)
        {
            var (total, items) = await _callRepository.QueryPagedAsync(
                x=> true,
                x => x.OrderByDescending(d => d.CreationTime),
                request.PageIndex,
                request.PageSize,
                (!string.IsNullOrEmpty(request.PhoneNum), d => d.FromNo.Contains(request.PhoneNum!))
            );
            return new PagedDto<CallDto>(total, _mapper.Map<IReadOnlyList<CallDto>>(items));
        }

        #endregion

        #region 对外通话记录

        /// <summary>
        /// 通话记录（外部对接）
        /// </summary>
        /// <param name="request"></param>
        /// <returns></returns>
        [HttpGet("out-calllist")]
        public async Task<IReadOnlyList<OutCallDto>> GetOutCallList([FromQuery] GetOutCallListRequest request)
        {
            var list = await _callRepository.QueryPartAsync(request.time);
            List<OutCallDto> outCallList = new List<OutCallDto>();
            foreach (var item in list)
            {
                OutCallDto outCallDto = new OutCallDto();
                outCallDto.CallId = item.Id;
                outCallDto.InfoType = Share.Enums.EInfoType.Call;//TODO目前写死（只有电话）
                outCallDto.Direction = item.CallDirection;
                outCallDto.Cpn = item.FromNo??"";
                outCallDto.Cdpn = item.ToNo ?? "";
                if (outCallDto.Direction == ECallDirection.In)
                {
                    outCallDto.Answered = item.CallDetails?.FirstOrDefault(x => x.EventName == "ANSWER")?.AnswerNo ?? "";
                    outCallDto.OnTime = item.CallDetails?.FirstOrDefault(x => x.EventName == "ANSWER")?.CreationTime;
                    outCallDto.OnState = item.CallDetails?.Any(x => x.EventName == "ANSWER") ==true ? EOnState.On : EOnState.NoOn;
                }
                else
                {
                    outCallDto.Answered = item.CallDetails?.FirstOrDefault(x => x.EventName == "ANSWERED")?.AnswerNo ?? "";
                    outCallDto.OnTime = item.CallDetails?.FirstOrDefault(x => x.EventName == "ANSWERED")?.CreationTime;//TODO 接通时间是否可以为空
                    outCallDto.OnState = item.CallDetails?.Any(x => x.EventName == "ANSWERED") == true ? EOnState.On : EOnState.NoOn;
                }
                outCallDto.BeginTime = item.CreationTime;
                outCallDto.ByeTime = item.CallDetails?.FirstOrDefault(x => x.EventName == "BYE")?.CreationTime;
                outCallDto.TalkTime = item.Duration;
                outCallDto.SoundFileName = item.CallDetails?.FirstOrDefault(x => x.EventName == "BYE")?.Recording ?? "";
                outCallDto.EvaluateResult = "";
                outCallDto.EndBy = item.EndBy;
                outCallList.Add(outCallDto);
            }
            return outCallList;
        }

        #endregion

    }
}
