﻿using CallCenter.Caches;
using CallCenter.Calls;
using CallCenter.Devices;
using CallCenter.Repository.SqlSugar;
using CallCenter.Share.Dtos;
using CallCenter.Share.Enums;
using CallCenter.Tels;
using CallCenter.Users;
using MapsterMapper;
using Microsoft.AspNetCore.Mvc;
using System.Transactions;
using XF.Domain.Authentications;
using XF.Domain.Cache;
using XF.Domain.Exceptions;
using XF.Utility.EnumExtensions;

namespace CallCenter.Api.Controllers
{
    /// <summary>
    /// 话机相关接口
    /// </summary>
    public class TelController : BaseController
    {
        private readonly ITelDomainService _telDomainService;
        private readonly IUserCacheManager _userCacheManager;
        private readonly ITelRepository _telRepository;
        private readonly ITelGroupRepository _telGroupRepository;
        private readonly IWorkRepository _workRepository;
        private readonly ISessionContext _sessionContext;
        private readonly ITypedCache<Tel> _cacheTel;
        private readonly ITypedCache<TelGroup> _cacheTelGroup;
        private readonly ITelCacheManager _telCacheManager;
        private readonly IDeviceManager _deviceManager;
        private readonly IMapper _mapper;
        private readonly ICallDetailRepository _callDetailRepository;
        private readonly ICallRepository _callRepository;

        public TelController(
            ITelDomainService telDomainService,
            IUserCacheManager userCacheManager,
            ITelRepository telRepository,
            ITelGroupRepository telGroupRepository,
            IWorkRepository workRepository,
            ISessionContext sessionContext,
            ITypedCache<Tel> cacheTel,
            ITypedCache<TelGroup> cacheTelGroup,
            ITelCacheManager telCacheManager,
            IDeviceManager deviceManager,
            IMapper mapper,
            ICallDetailRepository callDetailRepository,
            ICallRepository callRepository)
        {
            _telDomainService = telDomainService;
            _userCacheManager = userCacheManager;
            _telRepository = telRepository;
            _telGroupRepository = telGroupRepository;
            _workRepository = workRepository;
            _sessionContext = sessionContext;
            _cacheTel = cacheTel;
            _cacheTelGroup = cacheTelGroup;
            _telCacheManager = telCacheManager;
            _deviceManager = deviceManager;
            _mapper = mapper;
            _callDetailRepository = callDetailRepository;
            _callRepository = callRepository;
        }
        /// <summary>
        /// 查询所有分机
        /// </summary>
        /// <returns></returns>
        [HttpGet("tels")]
        public async Task<IReadOnlyList<TelDto>> QueryTels()
        {
            var tels = await _telRepository.QueryExtAsync(d => !d.IsDeleted, d => d.Includes(x => x.Groups));
            return _mapper.Map<IReadOnlyList<TelDto>>(tels);
        }

        /// <summary>
        /// 查询所有分机（高频）
        /// </summary>
        /// <returns></returns>
        [HttpGet("tels-frequency")]
        public async Task<IReadOnlyList<TelDto>> QueryTelsFrequency()
        {
            var tels = _telCacheManager.GetTels();
            var dtos = new List<TelDto>();
            foreach (var tel in tels)
            {
                var telModel = _mapper.Map<TelDto>(_telCacheManager.GetTel(tel.No));
                var telDto = await _deviceManager.QueryTelAsync(tel.No, HttpContext.RequestAborted);

                if (telDto!=null)
                {
                    telModel.CPN = telDto.CPN;
                    telModel.CDPN = telDto.CDPN;
                    telModel.TelStatusInfo = telDto.TelStatusInfo;
                    telModel.ConversationId = telDto.ConversationId;
                    if (telModel.TelStatus == Share.Enums.ETelStatus.Active)
                        telModel.TelStatus = telDto.TelStatus;

                    if (telModel.TelStatusInfo == ETelStatusInfo.Out)
                    {
                        var temp = await _callRepository.GetAsync(x => x.FromNo == telDto.CPN && x.ToNo == telDto.CDPN && x.CallDirection == ECallDirection.Out && x.ConversationId == telModel.ConversationId);
                        telModel.CallId = temp?.Id;
                    }
                    else if (telModel.TelStatusInfo == ETelStatusInfo.Into)
                    {
                        var temp = await _callRepository.GetAsync(x => x.FromNo == telDto.CPN && x.ToNo == telDto.CDPN && x.CallDirection == ECallDirection.In && x.ConversationId == telModel.ConversationId);
                        telModel.CallId = temp?.Id;
                    }
                }
                dtos.Add(telModel);
            }
            return dtos;
        }

        /// <summary>
        /// 查询当前用户分机（高频）
        /// </summary>
        /// <returns></returns>
        [HttpGet("tel-frequency")]
        public async Task<TelDto>  QueryTelByToken()
        {
            var userid = _sessionContext.RequiredUserId;
            var work = _userCacheManager.GetWorkByUser(userid);
            var telModel = _mapper.Map<TelDto>(_telCacheManager.GetTel(work.TelNo));
            var telDto = await _deviceManager.QueryTelAsync(work.TelNo, HttpContext.RequestAborted);

            if (telDto!=null)
            {
                telModel.CPN = telDto.CPN;
                telModel.CDPN = telDto.CDPN;
                telModel.TelStatusInfo = telDto.TelStatusInfo;
                telModel.ConversationId = telDto.ConversationId;
                if (telModel.TelStatus == Share.Enums.ETelStatus.Active)
                    telModel.TelStatus = telDto.TelStatus;

                if(telModel.TelStatusInfo== ETelStatusInfo.Out)
                {
                    var temp = await _callRepository.GetAsync(x => x.FromNo == telDto.CPN && x.ToNo == telDto.CDPN && x.CallDirection== ECallDirection.Out && x.ConversationId == telModel.ConversationId);
                    telModel.CallId = temp?.Id;
                }
                else if(telModel.TelStatusInfo == ETelStatusInfo.Into)
                {
                    var temp = await _callRepository.GetAsync(x => x.FromNo == telDto.CPN && x.CallDirection == ECallDirection.In && x.ConversationId == telModel.ConversationId);
                    telModel.CallId = temp?.Id;
                }
            }
            return telModel;
        }


        /// <summary>
        /// 查询所有分机组
        /// </summary>
        /// <returns></returns>
        [HttpGet("groups")]
        public async Task<IReadOnlyList<TelGroupDto>> QueryTelGroups()
        {
            var groups = await _telGroupRepository.QueryExtAsync(d => true, d => d.Includes(x => x.Tels));
            return _mapper.Map<IReadOnlyList<TelGroupDto>>(groups);
        }

        /// <summary>
        /// 新增分机组
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("group")]
        public async Task AddTelGroup([FromBody] AddTelGroupDto dto)
        {
            var works = await _workRepository.QueryAsync(d => dto.TelNos.Contains(d.TelNo) && !d.EndTime.HasValue);

            await _deviceManager.AssginConfigGroupAsync(
                dto.No,
                dto.Distribution.ToString().ToLower(),
                ext: works.Select(d => d.TelNo).ToList(),
                voiceFile: dto.Voice ?? null,
                cancellationToken: HttpContext.RequestAborted);

            var group = _mapper.Map<TelGroup>(dto);
            var tels = await _telRepository.QueryAsync(d => dto.TelNos.Contains(d.No));
            group.Tels = tels;
            await _telGroupRepository.AddNavTelsAsync(group, HttpContext.RequestAborted);
            _cacheTelGroup.Remove(dto.No);
        }

        /// <summary>
        /// 更新分机组
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPut("group")]
        public async Task UpdateTelGroup([FromBody] UpdateTelGroupDto dto)
        {
            var works = await _workRepository.QueryAsync(d => dto.TelNos.Contains(d.TelNo) && !d.EndTime.HasValue);

            await _deviceManager.AssginConfigGroupAsync(
                dto.No,
                dto.Distribution.ToString().ToLower(),
                ext: works.Select(d => d.TelNo).ToList(),
                voiceFile: dto.Voice ?? null,
                cancellationToken: HttpContext.RequestAborted);

            var group = _mapper.Map<TelGroup>(dto);
            var tels = await _telRepository.QueryAsync(d => dto.TelNos.Contains(d.No));
            group.Tels = tels;
            await _telGroupRepository.UpdateNavTelsAsync(group, HttpContext.RequestAborted);
            _cacheTelGroup.Remove(dto.No);
        }

        /// <summary>
        /// 分机休息
        /// </summary>
        /// <returns></returns>
        [HttpPut("rest")]
        public async Task Rest()
        {
            var work = _userCacheManager.GetWorkByUser(_sessionContext.RequiredUserId);
            if (work is null)
                throw new UserFriendlyException("当前坐席暂未进行工作");
            await _telDomainService.RestAsync(work, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 分机结束休息
        /// </summary>
        [HttpPut("unrest")]
        public async Task<TelRestDto> UnRest()
        {
            var work = _userCacheManager.GetWorkByUser(_sessionContext.RequiredUserId);
            if (work is null)
                throw new UserFriendlyException("当前坐席暂未进行工作");
            var telRest = await _telDomainService.UnRestAsync(work.TelId, HttpContext.RequestAborted);
            return _mapper.Map<TelRestDto>(telRest);
        }

        /// <summary>
        /// 保持通话
        /// </summary>
        [HttpPut("hold")]
        public async Task Hold()
        {
            var work = _userCacheManager.GetWorkByUser(_sessionContext.RequiredUserId);
            if (work is null)
                throw new UserFriendlyException("当前坐席暂未进行工作");
            await _telDomainService.HoldAsync(work.TelId, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 恢复通话(解除hold状态)
        /// </summary>
        [HttpPut("unhold")]
        public async Task UnHold()
        {
            var work = _userCacheManager.GetWorkByUser(_sessionContext.RequiredUserId);
            if (work is null)
                throw new UserFriendlyException("当前坐席暂未进行工作");
            await _telDomainService.UnHoldAsync(work.TelId, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 根据设备自动同步分机数据到数据库
        /// </summary>
        /// <returns></returns>
        [HttpPut("sync-tels")]
        public async Task SyncTelsAsync()
        {
            var dbTels = await _telRepository.QueryAsync();
            foreach (var dbTel in dbTels)
            {
                _cacheTel.Remove(dbTel.No);
            }
            //await _telRepository.RemoveRangeAsync(dbTels);
            var tels = await _telDomainService.QueryTelsAsync(HttpContext.RequestAborted);
            var list = tels.ExceptBy(dbTels.Select(e => e.No), x => x.No).ToList();
            var delList = dbTels.ExceptBy(tels.Select(e => e.No), x => x.No).ToList();

            for (int i = 0; i < delList.Count; i++)
            {
                delList[i].SoftDelete();
            }
            await _telRepository.UpdateRangeAsync(delList);
            await _telRepository.AddRangeAsync(list);
        }

        /// <summary>
        /// 页面基础信息
        /// </summary>
        /// <returns></returns>
        [HttpGet("base-info")]
        public async Task<dynamic> GetBaseInfo()
        {
            return new
            {
                Distributions = EnumExts.GetDescriptions<EDistribution>()
            };
        }

        /// <summary>
        /// 分机呼分机
        /// </summary>
        /// <returns></returns>
        [HttpPost("tel-to-tel")]
        public async Task TelToTel([FromBody] TelToTelDto dto)
        {
            var work = _userCacheManager.GetWorkByUser(_sessionContext.RequiredUserId);
            if (work is null)
                throw new UserFriendlyException("当前坐席暂未进行工作");
            await _deviceManager.ExtToExtAsync(work.TelNo, dto.TelNo, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 分机拨打外部电话
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("tel-to-outer")]
        public async Task TelToOuter([FromBody] TelToOuterDto dto)
        {
            var work = _userCacheManager.GetWorkByUser(_sessionContext.RequiredUserId);
            if (work is null)
                throw new UserFriendlyException("当前坐席暂未进行工作");
            await _deviceManager.ExtToOuterAsync(work.TelNo, dto.OuterNo, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 来电转分机
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("visitor-to-tel")]
        public async Task VisitorToTel([FromBody] VisitorToTelDto dto)
        {
            var work = _userCacheManager.GetWorkByUser(_sessionContext.RequiredUserId);
            if (work is null)
                throw new UserFriendlyException("当前坐席暂未进行工作");
            var tel = await _deviceManager.QueryTelAsync(work.TelNo, HttpContext.RequestAborted);
            if (!string.IsNullOrEmpty(tel.ConversationId))
                await _deviceManager.VisitorToExtAsync(tel.ConversationId, dto.TelNo, HttpContext.RequestAborted);
            else
                throw new UserFriendlyException("当前分机没有通话");
        }

        /// <summary>
        /// 来电转外部电话
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("visitor-to-outer")]
        public async Task VisitorToOuter([FromBody] VisitorToOuterDto dto)
        {
            var work = _userCacheManager.GetWorkByUser(_sessionContext.RequiredUserId);
            if (work is null)
                throw new UserFriendlyException("当前坐席暂未进行工作");
            var tel = await _deviceManager.QueryTelAsync(work.TelNo, HttpContext.RequestAborted);
            if (!string.IsNullOrEmpty(tel.ConversationId))
                await _deviceManager.VisitorToOuterAsync(tel.ConversationId, dto.OuterNo, HttpContext.RequestAborted);
            else
                throw new UserFriendlyException("当前分机没有通话");
        }


        /// <summary>
        /// 来电转分机组队列
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("visitor-to-group")]
        public async Task VisitorToGroup([FromBody] VisitorToGroupDto dto)
        {
            var work = _userCacheManager.GetWorkByUser(_sessionContext.RequiredUserId);
            if (work is null)
                throw new UserFriendlyException("当前坐席暂未进行工作");
            var tel = await _deviceManager.QueryTelAsync(work.TelNo, HttpContext.RequestAborted);
            if (!string.IsNullOrEmpty(tel.ConversationId))
                await _deviceManager.VisitorToGroupAsync(tel.ConversationId, dto.groupid, HttpContext.RequestAborted);
            else
                throw new UserFriendlyException("当前分机没有通话");
        }

        /// <summary>
        /// 去电转外部电话
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        /// <exception cref="UserFriendlyException"></exception>
        [HttpPost("outer-to-outer")]
        public async Task OuterToOuter([FromBody]OuterToOuterDto dto)
        {
            var work = _userCacheManager.GetWorkByUser(_sessionContext.RequiredUserId);
            if (work is null)
                throw new UserFriendlyException("当前坐席暂未进行工作");
            var tel = await _deviceManager.QueryTelAsync(work.TelNo, HttpContext.RequestAborted);
            if (!string.IsNullOrEmpty(tel.ConversationId))
                await _deviceManager.OuterToOuterAsync(tel.ConversationId, dto.OuterNo, HttpContext.RequestAborted);
            else
                throw new UserFriendlyException("当前分机没有通话");
        }

        /// <summary>
        /// 去电转分机
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        /// <exception cref="UserFriendlyException"></exception>
        [HttpPost("outer-to-tel")]
        public async Task OuterToTel([FromBody]OuterToTelDto dto)
        {
            var work = _userCacheManager.GetWorkByUser(_sessionContext.RequiredUserId);
            if (work is null)
                throw new UserFriendlyException("当前坐席暂未进行工作");
            var tel = await _deviceManager.QueryTelAsync(work.TelNo, HttpContext.RequestAborted);
            if (!string.IsNullOrEmpty(tel.ConversationId))
                await _deviceManager.OuterToExtAsync(tel.ConversationId, dto.TelNo, HttpContext.RequestAborted);
            else
                throw new UserFriendlyException("当前分机没有通话");
        }

        /// <summary>
        /// 三方会议
        /// 先建立两方通话，然后调用保持通话接口，拨通第三方分机，然后再调用三方会议接口
        /// 1. 分机 A 正在和 B 通话；
        /// 2. 分机 A 把原通话呼叫保持；
        /// 3. 分机 A 向 C 发起新的呼叫，并建立通话；
        /// 4. 此时，使用该 API 能够实现以分机 A 为主持方建立 A、B、C 的三方会议。
        /// </summary>
        /// <param name="dto">TelNo：会议发起方分机号</param>
        /// <returns></returns>
        /// <exception cref="UserFriendlyException"></exception>
        [HttpPost("meeting")]
        public async Task Conference([FromBody]ConferenceDto dto)
        {
            var work = _userCacheManager.GetWorkByUser(_sessionContext.RequiredUserId);
            if (work is null)
                throw new UserFriendlyException("当前坐席暂未进行工作");
            var tel = await _deviceManager.QueryTelAsync(work.TelNo, HttpContext.RequestAborted);
            if (!string.IsNullOrEmpty(tel.ConversationId))
                await _deviceManager.ConferenceMeetingAsync(dto.TelNo, HttpContext.RequestAborted);
            else
                throw new UserFriendlyException("当前分机没有通话");
        }

    }
}
