﻿using CallCenter.Application;
using CallCenter.Caches;
using CallCenter.Share.Dtos;
using CallCenter.Share.Requests;
using CallCenter.Tels;
using CallCenter.Users;
using MapsterMapper;
using Microsoft.AspNetCore.Mvc;
using XF.Domain.Authentications;
using XF.Domain.Cache;
using XF.Domain.Exceptions;

namespace CallCenter.Api.Controllers;

public class UserController : BaseController
{
    private readonly ISessionContext _sessionContext;
    private readonly IUserDomainService _userDomainService;
    private readonly ITelRepository _telRepository;
    private readonly IUserRepository _userRepository;
    private readonly ITelCacheManager _telCacheManager;
    private readonly IUserCacheManager _userCacheManager;
    private readonly IMapper _mapper;

    public UserController(
        ISessionContext sessionContext,
        IUserDomainService userDomainService,
        ITelRepository telRepository,
        IUserRepository userRepository,
        ITelCacheManager telCacheManager,
        IUserCacheManager userCacheManager,
        IMapper mapper)
    {
        _sessionContext = sessionContext;
        _userDomainService = userDomainService;
        _telRepository = telRepository;
        _userRepository = userRepository;
        _telCacheManager = telCacheManager;
        _userCacheManager = userCacheManager;
        _mapper = mapper;
    }

    /// <summary>
    /// 上班
    /// </summary>
    [HttpPost("on-duty/{telNo}")]
    public async Task OnDuty([FromRoute] string? telNo)
    {
        if (string.IsNullOrEmpty(telNo))
        {
            var user = await _userRepository.GetAsync(d => !d.IsDeleted && d.Id == _sessionContext.RequiredUserId,
                HttpContext.RequestAborted);
            if (user == null)
                throw new UserFriendlyException("无效用户编号");
            if (string.IsNullOrEmpty(user.DefaultTelNo))
                throw new UserFriendlyException("未设置默认分机号");
            telNo = user.DefaultTelNo;
        }
        var tel = _telCacheManager.GetTel(telNo);
        await _userDomainService.OnDutyAsync(_sessionContext.RequiredUserId, tel, HttpContext.RequestAborted);
    }

    /// <summary>
    /// 下班
    /// </summary>
    [HttpPost("off-duty")]
    public Task<WorkDto?> OffDuty()
    {
        return _userDomainService.OffDutyAsync(_sessionContext.RequiredUserId, HttpContext.RequestAborted);
    }

    /// <summary>
    /// 分页查询用户
    /// </summary>
    /// <returns></returns>
    [HttpGet("paged")]
    public async Task<PagedDto<UserDto>> QueryPaged([FromQuery] UserPagedRequest request)
    {
        var (total, items) = await _userRepository.QueryPagedAsync(
            d => !d.IsDeleted,
            d => d.OrderByDescending(x => x.CreationTime),
            request.PageIndex,
            request.PageSize,
            (!string.IsNullOrEmpty(request.PhoneNo), d => d.PhoneNo.Contains(request.PhoneNo!)),
            (!string.IsNullOrEmpty(request.Name), d => d.Name.Contains(request.Name!)),
            (!string.IsNullOrEmpty(request.NickName), d => !string.IsNullOrEmpty(d.NickName) && d.NickName.Contains(request.NickName!)));
        return new PagedDto<UserDto>(total, _mapper.Map<IReadOnlyList<UserDto>>(items));
    }

    /// <summary>
    /// 更新用户
    /// </summary>
    /// <param name="userDto"></param>
    /// <returns></returns>
    [HttpPut]
    public async Task Update([FromBody] UpdateUserDto userDto)
    {
        var user = _mapper.Map<User>(userDto);
        await _userRepository.UpdateAsync(user, HttpContext.RequestAborted);
    }

    /// <summary>
    /// 新增用户
    /// </summary>
    /// <param name="userDto"></param>
    /// <returns></returns>
    [HttpPost]
    public async Task<string> Add([FromBody] AddUserDto userDto)
    {
        var user = await _userRepository.GetAsync(d => d.PhoneNo == userDto.PhoneNo, HttpContext.RequestAborted);
        if (user is null)
        {
            user = _mapper.Map<User>(userDto);
            return await _userRepository.AddAsync(user, HttpContext.RequestAborted);
        }
        else if (user.IsDeleted)
        {
            user.Recover();
            await _userRepository.UpdateAsync(user);
            return user.Id;
        }
        else
        {
            throw new UserFriendlyException("该用户已存在");
        }
    }

    /// <summary>
    /// 删除用户
    /// </summary>
    /// <param name="id"></param>
    /// <returns></returns>
    [HttpDelete("{id}")]
    public async Task Remove(string id)
    {
        var work = _userCacheManager.GetWorkByUser(id);
        if (work is not null)
            throw new UserFriendlyException("该用户正在工作中，请下班以后再删除");
        await _userRepository.RemoveAsync(id, true, HttpContext.RequestAborted);
    }

    /// <summary>
    /// 查询用户当前状态
    /// </summary>
    /// <returns></returns>
    [HttpGet("state")]
    public async Task<UserStateDto> GetUserState()
    {
        var userId = _sessionContext.RequiredUserId;
        var isOnDuty = await _userCacheManager.IsWorkingByUserAsync(userId, HttpContext.RequestAborted);
        var isResting = false;
        var telNo = string.Empty;
        if (isOnDuty)
        {
            var work = _userCacheManager.GetWorkByUser(userId);
            isResting = await _telRepository.IsRestingAsync(work.TelNo, HttpContext.RequestAborted);
            telNo = work.TelNo;
        }

        return new UserStateDto(isOnDuty, isResting, telNo);
    }
}