﻿using System.ComponentModel;
using SqlSugar;

namespace XF.Domain;

public abstract class Entity : IEntity<string>, IDomainEvents
{
    private List<IAppNotification> _domainEvents = new();

    public string Id { get; set; }//todo 判断为空时生成递增Id

    public void AddEvent(IAppNotification notification)
    {
        if (_domainEvents.Contains(notification)) return;
        _domainEvents.Add(notification);
    }

    public void ClearEvents() => _domainEvents.Clear();

    public ICollection<IAppNotification> QueryAllEvents() => _domainEvents;
}

/// <summary>
/// 实体（带有创建时间）
/// </summary>
public abstract class CreationEntity : Entity, IHasCreationTime
{
    /// <summary>
    /// 创建时间
    /// </summary>
    [SugarColumn(ColumnDescription = "创建时间")]
    public DateTime CreationTime { get; set; }
}

/// <summary>
/// 实体（带有软删除）
/// </summary>
public abstract class SoftDeleteEntity : Entity, IHasDeletionTime, ISoftDelete
{
    /// <summary>
    /// 删除时间
    /// </summary>
    [SugarColumn(ColumnDescription = "删除时间")]
    public DateTime? DeletionTime { get; private set; }

    [SugarColumn(ColumnDescription = "是否删除")]
    public bool IsDeleted { get; private set; }

    public void SoftDelete()
    {
        IsDeleted = true;
        DeletionTime = DateTime.Now;
    }
}

/// <summary>
/// 实体（带有创建时间和软删除）
/// </summary>
public abstract class CreationSoftDeleteEntity : CreationEntity, IHasDeletionTime, ISoftDelete
{
    [SugarColumn(ColumnDescription = "是否删除")]
    public bool IsDeleted { get; private set; }

    /// <summary>
    /// 删除时间
    /// </summary>
    [SugarColumn(ColumnDescription = "删除时间")]
    public DateTime? DeletionTime { get; set; }

    public void SoftDelete()
    {
        IsDeleted = true;
        DeletionTime = DateTime.Now;
    }

    public void Recover() => IsDeleted = false;
}

public abstract class CreationModificationEntity : CreationEntity, IHasModificationTime
{
    /// <summary>
    /// 最近更新时间
    /// </summary>
    [SugarColumn(ColumnDescription = "最近更新时间")]
    public DateTime? LastModificationTime { get; set; }

    public void Modified() => LastModificationTime = DateTime.Now;
}

/// <summary>
/// 全状态实体(创建时间、更新时间、软删除)
/// </summary>
public abstract class FullStateEntity : CreationSoftDeleteEntity, IHasModificationTime
{
    /// <summary>
    /// 最近更新时间
    /// </summary>
    [SugarColumn(ColumnDescription = "最近更新时间")]
    public DateTime? LastModificationTime { get; set; }

    public void Modified() => LastModificationTime = DateTime.Now;
}